# flake8: noqa
# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
import datetime
from django.conf import settings


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Answer',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('value', models.CharField(max_length=256)),
                ('label', models.TextField(blank=True)),
                ('correct', models.BooleanField(default=False)),
                ('explanation', models.TextField(blank=True)),
            ],
            options={
                'ordering': ('question',),
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='Question',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('text', models.TextField()),
                ('question_type', models.CharField(max_length=256, choices=[(b'multiple choice', b'Multiple Choice: Multiple answers'), (b'single choice', b'Multiple Choice: Single answer'), (b'single choice dropdown', b'Multiple Choice: Single answer (dropdown)'), (b'short text', b'Short Text'), (b'long text', b'Long Text')])),
                ('explanation', models.TextField(blank=True)),
                ('intro_text', models.TextField(blank=True)),
            ],
            options={
                'ordering': ('quiz',),
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='Quiz',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('description', models.TextField(blank=True)),
                ('rhetorical', models.BooleanField(default=False)),
                ('allow_redo', models.BooleanField(default=True)),
                ('show_submit_state', models.BooleanField(default=True)),
            ],
            options={
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='Response',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('value', models.TextField(blank=True)),
                ('question', models.ForeignKey(to='quizblock.Question')),
            ],
            options={
                'ordering': ('question',),
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='Submission',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('submitted', models.DateTimeField(default=datetime.datetime.now)),
                ('quiz', models.ForeignKey(to='quizblock.Quiz')),
                ('user', models.ForeignKey(to=settings.AUTH_USER_MODEL)),
            ],
            options={
            },
            bases=(models.Model,),
        ),
        migrations.AddField(
            model_name='response',
            name='submission',
            field=models.ForeignKey(to='quizblock.Submission'),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='question',
            name='quiz',
            field=models.ForeignKey(to='quizblock.Quiz'),
            preserve_default=True,
        ),
        migrations.AlterOrderWithRespectTo(
            name='question',
            order_with_respect_to='quiz',
        ),
        migrations.AddField(
            model_name='answer',
            name='question',
            field=models.ForeignKey(to='quizblock.Question'),
            preserve_default=True,
        ),
        migrations.AlterOrderWithRespectTo(
            name='answer',
            order_with_respect_to='question',
        ),
    ]



from django.test import TestCase
from quizblock.models import Quiz, Question, Answer, Submission
from quizblock.models import Response
from django.contrib.auth.models import User


class FakeReq(object):
    def __init__(self):
        self.POST = dict()


class TestBasics(TestCase):
    def test_create(self):
        q = Quiz()
        self.assertNotEqual(q, None)

    def test_needs_submit(self):
        q = Quiz()
        self.assertTrue(q.needs_submit())
        q.rhetorical = True
        self.assertFalse(q.needs_submit())

    def test_add_form(self):
        f = Quiz.add_form()
        self.assertTrue('rhetorical' in f.fields)
        self.assertTrue('allow_redo' in f.fields)
        self.assertTrue('show_submit_state' in f.fields)

    def test_create_method(self):
        r = FakeReq()
        q = Quiz.create(r)
        self.assertEquals(q.description, '')
        self.assertEquals(q.display_name, 'Quiz')
        self.assertFalse(q.show_submit_state)
        self.assertFalse(q.rhetorical)
        self.assertFalse(q.allow_redo)

    def test_create_method_two(self):
        r = FakeReq()
        r.POST['show_submit_state'] = 'on'
        r.POST['rhetorical'] = 'on'
        r.POST['allow_redo'] = 'on'
        q = Quiz.create(r)
        self.assertEquals(q.description, '')
        self.assertEquals(q.display_name, 'Quiz')
        self.assertTrue(q.show_submit_state)
        self.assertTrue(q.rhetorical)
        self.assertTrue(q.allow_redo)

    def test_dict_roundtrip(self):
        q1 = Quiz(description="first", show_submit_state=False)
        d = q1.as_dict()
        q2 = Quiz(description="second")
        q2.import_from_dict(d)
        self.assertEquals(q2.description, "first")
        self.assertEquals(q1.allow_redo, q2.allow_redo)
        self.assertEquals(q1.rhetorical, q2.rhetorical)
        self.assertEquals(q1.show_submit_state, q2.show_submit_state)

    def test_create_from_dict(self):
        q = Quiz(description="first")
        d = q.as_dict()
        q2 = Quiz.create_from_dict(d)
        self.assertEquals(q2.description, "first")
        self.assertEquals(q.allow_redo, q2.allow_redo)
        self.assertEquals(q.rhetorical, q2.rhetorical)
        self.assertEquals(q.show_submit_state, q2.show_submit_state)

    def test_summary_render(self):
        q = Quiz(description="short")
        self.assertEqual(q.summary_render(), "short")
        q.description = ''.join(str(x) for x in range(75))
        expected = ('012345678910111213141516171819202122'
                    '2324252627282930313233343...')
        self.assertEqual(q.summary_render(), expected)

    def test_edit(self):
        q = Quiz()
        q.edit(dict(description='foo', rhetorical='1',
                    allow_redo='0', show_submit_state='on'), None)
        self.assertEqual(q.description, 'foo')
        self.assertEqual(q.rhetorical, '1')
        self.assertEqual(q.allow_redo, '0')
        self.assertTrue(q.show_submit_state)

    def test_edit_two(self):
        q = Quiz()
        q.edit(dict(description='foo'), None)
        self.assertEqual(q.description, 'foo')
        self.assertFalse(q.rhetorical)
        self.assertFalse(q.allow_redo)
        self.assertFalse(q.show_submit_state)

    def test_edit_form(self):
        # can't figure out how to test this one since it depends on
        # urls being set up

        # q = Quiz()
        # f = q.edit_form()
        # self.assertTrue('description' in f.fields)
        pass

    def test_redirect_to_self_on_submit(self):
        q = Quiz()
        self.assertTrue(q.redirect_to_self_on_submit())

        q2 = Quiz(show_submit_state=False)
        self.assertFalse(q2.redirect_to_self_on_submit())

    def test_add_question_form(self):
        q = Quiz()
        f = q.add_question_form(None)
        self.assertTrue('text' in f.fields)


class UserTests(TestCase):
    def setUp(self):
        self.u = User.objects.create(username="testuser")

    def test_submit(self):
        q = Quiz.objects.create()
        self.assertFalse(q.unlocked(self.u))
        q.submit(self.u, dict(foo='bar'))
        self.assertTrue(q.unlocked(self.u))
        q.clear_user_submissions(self.u)
        self.assertFalse(q.unlocked(self.u))


class QuestionTest(TestCase):
    def test_unicode(self):
        quiz = Quiz.objects.create()
        question = Question.objects.create(
            quiz=quiz, text="foo", question_type="long text")
        self.assertEqual(str(question), "foo")

    def test_display_number(self):
        quiz = Quiz.objects.create()
        question = Question.objects.create(
            quiz=quiz, text="foo", question_type="long text")
        self.assertEqual(question.display_number(), 1)

    def test_edit_form(self):
        quiz = Quiz.objects.create()
        question = Question.objects.create(
            quiz=quiz, text="foo", question_type="long text")
        f = question.edit_form()
        self.assertTrue('question_type' in f.fields)

    def test_add_answer_form(self):
        quiz = Quiz.objects.create()
        question = Question.objects.create(
            quiz=quiz, text="foo", question_type="long text")
        f = question.add_answer_form()
        self.assertTrue(hasattr(f, 'fields'))

    def test_correct_answer_values(self):
        quiz = Quiz.objects.create()
        question = Question.objects.create(
            quiz=quiz, text="foo", question_type="long text")
        self.assertEqual(question.correct_answer_values(), [])

    def test_correct_answer_number_wrong_type(self):
        quiz = Quiz.objects.create()
        question = Question.objects.create(
            quiz=quiz, text="foo", question_type="long text")
        self.assertEqual(question.correct_answer_number(), None)

    def test_correct_answer_number(self):
        quiz = Quiz.objects.create()
        question = Question.objects.create(
            quiz=quiz, text="foo", question_type="single choice")
        Answer.objects.create(question=question, value='1', label='one',
                              correct=True)
        self.assertEqual(question.correct_answer_number(), 0)

    def test_correct_answer_letter_wrong_type(self):
        quiz = Quiz.objects.create()
        question = Question.objects.create(
            quiz=quiz, text="foo", question_type="long text")
        self.assertEqual(question.correct_answer_letter(), None)

    def test_correct_answer_letter(self):
        quiz = Quiz.objects.create()
        question = Question.objects.create(
            quiz=quiz, text="foo", question_type="single choice")
        Answer.objects.create(question=question, value='1', label='one',
                              correct=True)
        self.assertEqual(question.correct_answer_letter(), 'A')

    def test_update_answers_order(self):
        quiz = Quiz.objects.create()
        question = Question.objects.create(
            quiz=quiz, text="foo", question_type="long text")
        question.update_answers_order([])

    def test_answerable(self):
        quiz = Quiz.objects.create()
        question = Question.objects.create(
            quiz=quiz, text="foo", question_type="long text")
        self.assertFalse(question.answerable())
        question = Question.objects.create(
            quiz=quiz, text="foo", question_type="single choice")
        self.assertTrue(question.answerable())

    def test_types(self):
        quiz = Quiz.objects.create()
        question = Question.objects.create(
            quiz=quiz, text="foo", question_type="long text")
        self.assertFalse(question.is_short_text())
        self.assertTrue(question.is_long_text())
        self.assertFalse(question.is_single_choice())
        self.assertFalse(question.is_single_choice_dropdown())
        self.assertFalse(question.is_multiple_choice())

    def test_as_dict(self):
        quiz = Quiz.objects.create()
        question = Question.objects.create(
            quiz=quiz, text="foo", question_type="long text")
        d = question.as_dict()
        self.assertEqual(d['text'], "foo")
        self.assertEqual(d['question_type'], "long text")

    def test_user_responses(self):
        user = User.objects.create(username="testuser")
        quiz = Quiz.objects.create()
        q1 = Question.objects.create(quiz=quiz, text="question_one",
                                     question_type="single choice")
        Answer.objects.create(question=q1, label="a", value="a", correct=True)
        Answer.objects.create(question=q1, label="b", value="b")

        self.assertEquals(len(q1.user_responses(user)), 0)

        s = Submission.objects.create(quiz=quiz, user=user)
        self.assertEquals(len(q1.user_responses(user)), 0)

        Response.objects.create(question=q1, submission=s, value="a")
        self.assertEquals(len(q1.user_responses(user)), 1)


class TestIsUserCorrect(TestCase):

    def setUp(self):
        self.user = User.objects.create(username="testuser")
        self.quiz = Quiz.objects.create()

    def test_no_questions(self):
        self.assertEquals(self.quiz.score(self.user), None)

    def test_short_text(self):
        question = Question.objects.create(quiz=self.quiz,
                                           text="question_one",
                                           question_type="short text")

        # no submissions
        self.assertIsNone(question.is_user_correct(self.user))
        self.assertEquals(self.quiz.score(self.user), None)

        sub = Submission.objects.create(quiz=self.quiz, user=self.user)
        Response.objects.create(question=question, submission=sub, value="a")

        self.assertTrue(question.is_user_correct(self.user))

        #score
        self.assertEquals(self.quiz.score(self.user), 1)

    def test_long_text(self):
        question = Question.objects.create(quiz=self.quiz,
                                           text="question_one",
                                           question_type="long text")

        # no submissions
        self.assertIsNone(question.is_user_correct(self.user))
        self.assertEquals(self.quiz.score(self.user), None)

        sub = Submission.objects.create(quiz=self.quiz, user=self.user)
        Response.objects.create(question=question, submission=sub, value="b")

        self.assertTrue(question.is_user_correct(self.user))
        self.assertEquals(self.quiz.score(self.user), 1)

    def test_single_choice_no_correct_answers(self):
        question = Question.objects.create(quiz=self.quiz,
                                           text="question_one",
                                           question_type="single choice")
        Answer.objects.create(question=question, label="a", value="a")
        Answer.objects.create(question=question, label="b", value="b")
        Answer.objects.create(question=question, label="c", value="c")

        # no response
        self.assertIsNone(question.is_user_correct(self.user))
        self.assertEquals(self.quiz.score(self.user), None)

        # user responded
        sub = Submission.objects.create(quiz=self.quiz, user=self.user)
        Response.objects.create(question=question, submission=sub, value="b")

        self.assertTrue(question.is_user_correct(self.user))
        self.assertEquals(self.quiz.score(self.user), 1)

    def test_single_choice_correct_answers(self):
        question = Question.objects.create(quiz=self.quiz,
                                           text="question_one",
                                           question_type="single choice")
        Answer.objects.create(question=question, label="a", value="a",
                              correct=True)
        Answer.objects.create(question=question, label="b", value="b")
        Answer.objects.create(question=question, label="c", value="c")

        # no response
        self.assertFalse(question.is_user_correct(self.user))
        self.assertEquals(self.quiz.score(self.user), None)

        # user responded
        sub = Submission.objects.create(quiz=self.quiz, user=self.user)
        response = Response.objects.create(question=question,
                                           submission=sub, value="a")

        self.assertTrue(question.is_user_correct(self.user))
        self.assertEquals(self.quiz.score(self.user), 1)

        response.value = 'b'
        response.save()
        self.assertFalse(question.is_user_correct(self.user))
        self.assertEquals(self.quiz.score(self.user), 0)

    def test_multiple_choice_correct_answers(self):
        question = Question.objects.create(quiz=self.quiz,
                                           text="question_one",
                                           question_type="multiple choice")
        Answer.objects.create(question=question, label="a", value="a",
                              correct=True)
        Answer.objects.create(question=question, label="b", value="b",
                              correct=True)
        Answer.objects.create(question=question, label="c", value="c")

        # no response
        self.assertFalse(question.is_user_correct(self.user))
        self.assertEquals(self.quiz.score(self.user), None)

        # user responded - incorrectly
        sub = Submission.objects.create(quiz=self.quiz, user=self.user)
        c = Response.objects.create(question=question,
                                    submission=sub, value="c")
        self.assertFalse(question.is_user_correct(self.user))
        self.assertEquals(self.quiz.score(self.user), 0)

        # user responded - partially incorrectly
        Response.objects.create(question=question, submission=sub, value="a")
        self.assertFalse(question.is_user_correct(self.user))
        self.assertEquals(self.quiz.score(self.user), 0)

        # user responded - partially incorrectly
        Response.objects.create(question=question, submission=sub, value="b")
        self.assertFalse(question.is_user_correct(self.user))
        self.assertEquals(self.quiz.score(self.user), 0)

        # kill the incorrect one
        c.delete()
        self.assertTrue(question.is_user_correct(self.user))
        self.assertEquals(self.quiz.score(self.user), 1)


class AnswerTest(TestCase):
    def test_unicode(self):
        quiz = Quiz.objects.create()
        question = Question.objects.create(
            quiz=quiz, text="foo", question_type="single choice")
        answer = Answer.objects.create(question=question, label="an answer")
        self.assertEqual(str(answer), "an answer")

    def test_edit_form(self):
        quiz = Quiz.objects.create()
        question = Question.objects.create(
            quiz=quiz, text="foo", question_type="single choice")
        answer = Answer.objects.create(question=question, label="an answer")
        f = answer.edit_form()
        self.assertTrue('label' in f.fields)

    def test_as_dict(self):
        quiz = Quiz.objects.create()
        question = Question.objects.create(
            quiz=quiz, text="foo", question_type="single choice")
        answer = Answer.objects.create(question=question, label="an answer")
        d = answer.as_dict()
        self.assertEqual(d['label'], "an answer")
        self.assertFalse(d['correct'])

    def test_quiz_round_trip(self):
        quiz = Quiz.objects.create()
        question = Question.objects.create(
            quiz=quiz, text="foo", question_type="single choice")
        Answer.objects.create(question=question, label="an answer")
        Answer.objects.create(
            question=question, label="another answer",
            explanation="an explanation")
        d = quiz.as_dict()
        quiz2 = Quiz.objects.create()
        quiz2.import_from_dict(d)
        self.assertEqual(quiz2.question_set.count(), 1)


class SubmissionTest(TestCase):
    def test_unicode(self):
        quiz = Quiz.objects.create()
        user = User.objects.create(username="testuser")
        s = Submission.objects.create(quiz=quiz, user=user)
        self.assertTrue(
            str(s).startswith("quiz %d submission by testuser" % quiz.id))


class ResponseTest(TestCase):
    def test_unicode(self):
        quiz = Quiz.objects.create()
        question = Question.objects.create(
            quiz=quiz, text="foo", question_type="single choice")
        Answer.objects.create(question=question, label="an answer")
        user = User.objects.create(username="testuser")
        s = Submission.objects.create(quiz=quiz, user=user)
        response = Response.objects.create(
            question=question, submission=s, value="an answer")
        self.assertTrue(str(response).startswith("response to "))

    def test_is_correct(self):
        quiz = Quiz.objects.create()
        question = Question.objects.create(
            quiz=quiz, text="foo", question_type="single choice")
        Answer.objects.create(question=question, label="an answer")
        user = User.objects.create(username="testuser")
        s = Submission.objects.create(quiz=quiz, user=user)
        response = Response.objects.create(
            question=question, submission=s, value="an answer")
        self.assertFalse(response.is_correct())



from datetime import datetime
from django import forms
from django.contrib.auth.models import User
from django.contrib.contenttypes import generic
from django.core.urlresolvers import reverse
from django.db import models
from django.utils.encoding import smart_str
from pagetree.models import PageBlock
from pagetree.reports import ReportableInterface, ReportColumnInterface


class Quiz(models.Model):
    pageblocks = generic.GenericRelation(PageBlock)
    description = models.TextField(blank=True)
    rhetorical = models.BooleanField(default=False)
    allow_redo = models.BooleanField(default=True)
    show_submit_state = models.BooleanField(default=True)
    template_file = "quizblock/quizblock.html"

    display_name = "Quiz"
    exportable = True
    importable = True

    def pageblock(self):
        return self.pageblocks.all()[0]

    def __unicode__(self):
        return unicode(self.pageblock())

    def needs_submit(self):
        return not self.rhetorical

    def submit(self, user, data):
        """ a big open question here is whether we should
        be validating submitted answers here, on submission,
        or let them submit whatever garbage they want and only
        worry about it when we show the admins the results """
        s = Submission.objects.create(quiz=self, user=user)
        for k in data.keys():
            if k.startswith('question'):
                qid = int(k[len('question'):])
                question = Question.objects.get(id=qid)
                # it might make more sense to just accept a QueryDict
                # instead of a dict so we can use getlist()
                if isinstance(data[k], list):
                    for v in data[k]:
                        Response.objects.create(
                            submission=s,
                            question=question,
                            value=v)
                else:
                    Response.objects.create(
                        submission=s,
                        question=question,
                        value=data[k])

    def redirect_to_self_on_submit(self):
        return self.show_submit_state

    def unlocked(self, user):
        # meaning that the user can proceed *past* this one,
        # not that they can access this one. careful.
        return Submission.objects.filter(quiz=self, user=user).count() > 0

    def edit_form(self):
        class EditForm(forms.Form):
            description = forms.CharField(widget=forms.widgets.Textarea(),
                                          initial=self.description)
            rhetorical = forms.BooleanField(initial=self.rhetorical)
            allow_redo = forms.BooleanField(initial=self.allow_redo)
            show_submit_state = forms.BooleanField(
                initial=self.show_submit_state)
            alt_text = ("<a href=\"" + reverse("edit-quiz", args=[self.id])
                        + "\">manage questions/answers</a>")
        return EditForm()

    @classmethod
    def add_form(self):
        class AddForm(forms.Form):
            description = forms.CharField(widget=forms.widgets.Textarea())
            rhetorical = forms.BooleanField()
            allow_redo = forms.BooleanField()
            show_submit_state = forms.BooleanField(initial=True)
        return AddForm()

    @classmethod
    def create(self, request):
        return Quiz.objects.create(
            description=request.POST.get('description', ''),
            rhetorical=request.POST.get('rhetorical', ''),
            allow_redo=request.POST.get('allow_redo', ''),
            show_submit_state=request.POST.get('show_submit_state', False))

    @classmethod
    def create_from_dict(self, d):
        q = Quiz.objects.create(
            description=d.get('description', ''),
            rhetorical=d.get('rhetorical', False),
            allow_redo=d.get('allow_redo', True),
            show_submit_state=d.get('show_submit_state', True)
        )
        q.import_from_dict(d)
        return q

    def edit(self, vals, files):
        self.description = vals.get('description', '')
        self.rhetorical = vals.get('rhetorical', '')
        self.allow_redo = vals.get('allow_redo', '')
        self.show_submit_state = vals.get('show_submit_state', False)
        self.save()

    def add_question_form(self, request=None):
        return QuestionForm(request)

    def update_questions_order(self, question_ids):
        self.set_question_order(question_ids)

    def clear_user_submissions(self, user):
        Submission.objects.filter(user=user, quiz=self).delete()

    def as_dict(self):
        d = dict(description=self.description,
                 rhetorical=self.rhetorical,
                 allow_redo=self.allow_redo,
                 show_submit_state=self.show_submit_state)
        d['questions'] = [q.as_dict() for q in self.question_set.all()]
        return d

    def import_from_dict(self, d):
        self.description = d['description']
        self.rhetorical = d['rhetorical']
        self.allow_redo = d.get('allow_redo', True)
        self.show_submit_state = d.get('show_submit_state', True)
        self.save()
        self.submission_set.all().delete()
        self.question_set.all().delete()
        for q in d['questions']:
            question = Question.objects.create(
                quiz=self, text=q['text'],
                question_type=q['question_type'],
                explanation=q['explanation'],
                intro_text=q['intro_text'])
            for a in q['answers']:
                x = Answer.objects.create(question=question,
                                          value=a['value'],
                                          label=a['label'],
                                          correct=a['correct'])
                if 'explanation' in a:
                    x.explanation = a['explanation']
                    x.save()

    def summary_render(self):
        if len(self.description) < 61:
            return self.description
        else:
            return self.description[:61] + "..."

    def report_metadata(self):
        columns = []
        hierarchy = self.pageblock().section.hierarchy
        for q in self.question_set.all():
            if q.answerable():
                # one row for each question/answer for choice questions
                for a in q.answer_set.all():
                    columns.append(QuestionColumn(hierarchy=hierarchy,
                                                  question=q, answer=a))
            else:
                columns.append(QuestionColumn(hierarchy=hierarchy, question=q))

        return columns

    def report_values(self):
        columns = []
        hierarchy = self.pageblock().section.hierarchy
        for q in self.question_set.all():
            if q.is_multiple_choice():
                # need to make a column for each answer
                for a in q.answer_set.all():
                    columns.append(QuestionColumn(
                        hierarchy=hierarchy, question=q, answer=a))
            else:
                # single choice, short text and long text need only one row
                columns.append(QuestionColumn(hierarchy=hierarchy, question=q))

        return columns

    def score(self, user):
        '''
            returns
            - None if incomplete or has no questions
            - float score if complete
        '''
        if self.question_set.count() == 0:
            return None

        score = 0.0
        for question in self.question_set.all():
            correct = question.is_user_correct(user)
            if correct is None:
                return None
            elif correct:
                score += 1

        return score / self.question_set.count()

ReportableInterface.register(Quiz)


class Question(models.Model):
    quiz = models.ForeignKey(Quiz)
    text = models.TextField()
    question_type = models.CharField(
        max_length=256,
        choices=(
            ("multiple choice", "Multiple Choice: Multiple answers"),
            ("single choice", "Multiple Choice: Single answer"),
            ("single choice dropdown",
             "Multiple Choice: Single answer (dropdown)"),
            ("short text", "Short Text"),
            ("long text", "Long Text"),
        ))
    explanation = models.TextField(blank=True)
    intro_text = models.TextField(blank=True)

    class Meta:
        ordering = ('quiz',)
        order_with_respect_to = 'quiz'

    def __unicode__(self):
        return self.text

    def display_number(self):
        return self._order + 1

    def edit_form(self, request=None):
        return QuestionForm(request, instance=self)

    def add_answer_form(self, request=None):
        return AnswerForm(request)

    def correct_answer_values(self):
        return [a.value for a in self.answer_set.filter(correct=True)]

    def correct_answer_number(self):
        if self.question_type != "single choice":
            return None
        return self.answer_set.filter(correct=True)[0]._order

    def correct_answer_letter(self):
        if (self.question_type != "single choice"
                or self.answer_set.count() == 0):
            return None
        return chr(ord('A') + self.correct_answer_number())

    def update_answers_order(self, answer_ids):
        self.set_answer_order(answer_ids)

    def answerable(self):
        """ whether it makes sense to have Answers associated with this """
        return self.question_type in ["multiple choice",
                                      "single choice",
                                      "single choice dropdown"]

    def is_short_text(self):
        return self.question_type == "short text"

    def is_long_text(self):
        return self.question_type == "long text"

    def is_single_choice(self):
        return self.question_type == "single choice"

    def is_single_choice_dropdown(self):
        return self.question_type == "single choice dropdown"

    def is_multiple_choice(self):
        return self.question_type == "multiple choice"

    def user_responses(self, user):
        qs = Submission.objects.filter(user=user,
                                       quiz=self.quiz).order_by("-submitted")
        if len(qs) == 0:
            return Response.objects.none()
        else:
            submission = qs[0]
            return Response.objects.filter(question=self,
                                           submission=submission)

    def as_dict(self):
        return dict(
            text=self.text,
            question_type=self.question_type,
            explanation=self.explanation,
            intro_text=self.intro_text,
            answers=[a.as_dict() for a in self.answer_set.all()]
        )

    def is_user_correct(self, user):
        '''
            * None if user has not responded
            * False if user has not answered or not answered correctly
            * True if user has responded and
                there is no correct answer or has answered correctly
        '''
        responses = self.user_responses(user)
        answers = self.correct_answer_values()

        if len(responses) == 0:
            return None  # incomplete

        if not self.answerable() or len(answers) == 0:
            return True  # a completed response is considered correct

        if len(answers) != len(responses):
            # The user hasn't completely answered the question yet
            return False

        correct = True
        for resp in responses:
            correct = correct and str(resp.value) in answers
        return correct


class Answer(models.Model):
    question = models.ForeignKey(Question)
    value = models.CharField(max_length=256)
    label = models.TextField(blank=True)
    correct = models.BooleanField(default=False)
    explanation = models.TextField(blank=True)

    class Meta:
        ordering = ('question',)
        order_with_respect_to = 'question'

    def __unicode__(self):
        return self.label

    def display_number(self):
        return self._order + 1

    def edit_form(self, request=None):
        return AnswerForm(request, instance=self)

    def as_dict(self):
        return dict(value=self.value,
                    label=self.label,
                    correct=self.correct,
                    explanation=self.explanation)


class Submission(models.Model):
    quiz = models.ForeignKey(Quiz)
    user = models.ForeignKey(User)
    submitted = models.DateTimeField(default=datetime.now)

    def __unicode__(self):
        return "quiz %d submission by %s at %s" % (self.quiz.id,
                                                   unicode(self.user),
                                                   self.submitted)


class Response(models.Model):
    question = models.ForeignKey(Question)
    submission = models.ForeignKey(Submission)
    value = models.TextField(blank=True)

    class Meta:
        ordering = ('question',)

    def __unicode__(self):
        return "response to %s [%s]" % (unicode(self.question),
                                        unicode(self.submission))

    def is_correct(self):
        return self.value in self.question.correct_answer_values()


class QuestionForm(forms.ModelForm):
    class Meta:
        model = Question
        exclude = ("quiz",)
        fields = ('question_type', 'intro_text', 'text', 'explanation')


class AnswerForm(forms.ModelForm):
    class Meta:
        model = Answer
        exclude = ("question",)

    def clean(self):
        if 'value' not in self.cleaned_data:
            raise forms.ValidationError(
                'Please enter a meaningful value for this answer.')
        else:
            return self.cleaned_data


class QuestionColumn(ReportColumnInterface):

    def __init__(self, hierarchy, question, answer=None):
        self.hierarchy = hierarchy
        self.question = question
        self.answer = answer

        self._submission_cache = Submission.objects.filter(
            quiz=self.question.quiz)
        self._response_cache = Response.objects.filter(
            question=self.question)
        self._answer_cache = self.question.answer_set.all()

    @classmethod
    def clean_header(cls, s):
        s = s.replace('<p>', '')
        s = s.replace('</p>', '')
        s = s.replace('</div>', '')
        s = s.replace('\n', '')
        s = s.replace('\r', '')
        s = s.replace('<', '')
        s = s.replace('>', '')
        s = s.replace('\'', '')
        s = s.replace('\"', '')
        s = s.replace(',', '')
        s = s.encode('utf-8')
        return s

    def question_id(self):
        return "%s_%s" % (self.hierarchy.id, self.question.id)

    def question_answer_id(self):
        return "%s_%s_%s" % (self.hierarchy.id,
                             self.question.id,
                             self.answer.id)

    def identifier(self):
        if self.question and self.answer:
            return self.question_answer_id()
        else:
            return self.question_id()

    def metadata(self):
        row = [self.hierarchy.name,
               self.question_id(),
               "Quiz",
               self.question.question_type,
               self.clean_header(self.question.text)]
        if self.answer:
            row.append(self.answer.id)
            row.append(self.clean_header(self.answer.label))
        return row

    def user_value(self, user):
        value = ''
        r = self._submission_cache.filter(user=user).order_by("-submitted")
        if r.count() == 0:
            # user has not answered this question
            return None
        submission = r[0]
        r = self._response_cache.filter(submission=submission)
        if r.count() > 0:
            if (self.question.is_short_text() or
                    self.question.is_long_text()):
                value = r[0].value
            elif self.question.is_multiple_choice():
                if self.answer.value in [res.value for res in r]:
                    value = self.answer.id
            else:  # single choice
                for a in self._answer_cache:
                    if a.value == r[0].value:
                        value = a.id
                        break

        return smart_str(value)



# encoding: utf-8
# flake8: noqa
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models

class Migration(SchemaMigration):

    def forwards(self, orm):
        
        # Adding model 'Quiz'
        db.create_table('quizblock_quiz', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('description', self.gf('django.db.models.fields.TextField')(blank=True)),
            ('rhetorical', self.gf('django.db.models.fields.BooleanField')(default=False)),
            ('allow_redo', self.gf('django.db.models.fields.BooleanField')(default=True)),
        ))
        db.send_create_signal('quizblock', ['Quiz'])

        # Adding model 'Question'
        db.create_table('quizblock_question', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('quiz', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['quizblock.Quiz'])),
            ('text', self.gf('django.db.models.fields.TextField')()),
            ('question_type', self.gf('django.db.models.fields.CharField')(max_length=256)),
            ('explanation', self.gf('django.db.models.fields.TextField')(blank=True)),
            ('intro_text', self.gf('django.db.models.fields.TextField')(blank=True)),
            ('_order', self.gf('django.db.models.fields.IntegerField')(default=0)),
        ))
        db.send_create_signal('quizblock', ['Question'])

        # Adding model 'Answer'
        db.create_table('quizblock_answer', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('question', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['quizblock.Question'])),
            ('value', self.gf('django.db.models.fields.CharField')(max_length=256, blank=True)),
            ('label', self.gf('django.db.models.fields.TextField')(blank=True)),
            ('correct', self.gf('django.db.models.fields.BooleanField')(default=False)),
            ('_order', self.gf('django.db.models.fields.IntegerField')(default=0)),
        ))
        db.send_create_signal('quizblock', ['Answer'])

        # Adding model 'Submission'
        db.create_table('quizblock_submission', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('quiz', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['quizblock.Quiz'])),
            ('user', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['auth.User'])),
            ('submitted', self.gf('django.db.models.fields.DateTimeField')(default=datetime.datetime.now)),
        ))
        db.send_create_signal('quizblock', ['Submission'])

        # Adding model 'Response'
        db.create_table('quizblock_response', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('question', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['quizblock.Question'])),
            ('submission', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['quizblock.Submission'])),
            ('value', self.gf('django.db.models.fields.TextField')(blank=True)),
        ))
        db.send_create_signal('quizblock', ['Response'])


    def backwards(self, orm):
        
        # Deleting model 'Quiz'
        db.delete_table('quizblock_quiz')

        # Deleting model 'Question'
        db.delete_table('quizblock_question')

        # Deleting model 'Answer'
        db.delete_table('quizblock_answer')

        # Deleting model 'Submission'
        db.delete_table('quizblock_submission')

        # Deleting model 'Response'
        db.delete_table('quizblock_response')


    models = {
        'auth.group': {
            'Meta': {'object_name': 'Group'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '80'}),
            'permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'})
        },
        'auth.permission': {
            'Meta': {'ordering': "('content_type__app_label', 'content_type__model', 'codename')", 'unique_together': "(('content_type', 'codename'),)", 'object_name': 'Permission'},
            'codename': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['contenttypes.ContentType']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        },
        'auth.user': {
            'Meta': {'object_name': 'User'},
            'date_joined': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'email': ('django.db.models.fields.EmailField', [], {'max_length': '75', 'blank': 'True'}),
            'first_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'groups': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Group']", 'symmetrical': 'False', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_active': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'is_staff': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'is_superuser': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'last_login': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'last_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'password': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'user_permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'}),
            'username': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '30'})
        },
        'contenttypes.contenttype': {
            'Meta': {'ordering': "('name',)", 'unique_together': "(('app_label', 'model'),)", 'object_name': 'ContentType', 'db_table': "'django_content_type'"},
            'app_label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        'pagetree.hierarchy': {
            'Meta': {'object_name': 'Hierarchy'},
            'base_url': ('django.db.models.fields.CharField', [], {'default': "''", 'max_length': '256'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '256'})
        },
        'pagetree.pageblock': {
            'Meta': {'ordering': "('section', 'ordinality')", 'object_name': 'PageBlock'},
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['contenttypes.ContentType']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'label': ('django.db.models.fields.CharField', [], {'max_length': '256', 'null': 'True', 'blank': 'True'}),
            'object_id': ('django.db.models.fields.PositiveIntegerField', [], {}),
            'ordinality': ('django.db.models.fields.PositiveIntegerField', [], {'default': '1'}),
            'section': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['pagetree.Section']"})
        },
        'pagetree.section': {
            'Meta': {'object_name': 'Section'},
            'depth': ('django.db.models.fields.PositiveIntegerField', [], {}),
            'hierarchy': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['pagetree.Hierarchy']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'label': ('django.db.models.fields.CharField', [], {'max_length': '256'}),
            'numchild': ('django.db.models.fields.PositiveIntegerField', [], {'default': '0'}),
            'path': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '255'}),
            'slug': ('django.db.models.fields.SlugField', [], {'max_length': '50', 'db_index': 'True'})
        },
        'quizblock.answer': {
            'Meta': {'ordering': "('_order',)", 'object_name': 'Answer'},
            '_order': ('django.db.models.fields.IntegerField', [], {'default': '0'}),
            'correct': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'label': ('django.db.models.fields.TextField', [], {'blank': 'True'}),
            'question': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['quizblock.Question']"}),
            'value': ('django.db.models.fields.CharField', [], {'max_length': '256', 'blank': 'True'})
        },
        'quizblock.question': {
            'Meta': {'ordering': "('_order',)", 'object_name': 'Question'},
            '_order': ('django.db.models.fields.IntegerField', [], {'default': '0'}),
            'explanation': ('django.db.models.fields.TextField', [], {'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'intro_text': ('django.db.models.fields.TextField', [], {'blank': 'True'}),
            'question_type': ('django.db.models.fields.CharField', [], {'max_length': '256'}),
            'quiz': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['quizblock.Quiz']"}),
            'text': ('django.db.models.fields.TextField', [], {})
        },
        'quizblock.quiz': {
            'Meta': {'object_name': 'Quiz'},
            'allow_redo': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'description': ('django.db.models.fields.TextField', [], {'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'rhetorical': ('django.db.models.fields.BooleanField', [], {'default': 'False'})
        },
        'quizblock.response': {
            'Meta': {'ordering': "('question',)", 'object_name': 'Response'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'question': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['quizblock.Question']"}),
            'submission': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['quizblock.Submission']"}),
            'value': ('django.db.models.fields.TextField', [], {'blank': 'True'})
        },
        'quizblock.submission': {
            'Meta': {'object_name': 'Submission'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'quiz': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['quizblock.Quiz']"}),
            'submitted': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'user': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['auth.User']"})
        }
    }

    complete_apps = ['quizblock']



from django.test import TestCase
from django.test.client import RequestFactory
from pagetree.tests.factories import UserFactory
from quizblock.models import Quiz, Question, Answer
from quizblock.views import EditQuizView, AddQuestionToQuizView, \
    EditQuestionView, AddAnswerToQuestionView, DeleteQuestionView, \
    DeleteAnswerView, EditAnswerView, ReorderAnswersView, ReorderQuestionsView


class LoggedInViewTest(TestCase):
    urls = 'quizblock.urls'

    def setUp(self):
        self.factory = RequestFactory()
        self.user = UserFactory()

        self.quiz = Quiz()
        self.quiz.save()

        self.single_answer = Question.objects.create(
            quiz=self.quiz, text='single answer',
            question_type='single choice')
        self.single_answer_one = Answer.objects.create(
            question=self.single_answer, label='Yes', value='1')
        self.single_answer_two = Answer.objects.create(
            question=self.single_answer, label='No', value='0')

    def test_edit_quiz(self):
        url = "/quizblock/edit_quiz/%s/" % self.quiz.id
        request = self.factory.get(url)
        request.user = self.user

        response = EditQuizView.as_view()(request, pk=self.quiz.id)
        self.assertEqual(response.status_code, 200)

    def test_add_question_to_quiz(self):
        self.assertEquals(self.quiz.question_set.count(), 1)
        url = "/quizblock/edit_quiz/%s/add_question" % self.quiz.id

        data = {u'text': [u'the text'],
                u'intro_text': [u'the intro'],
                u'explanation': [u'the explanation'],
                u'question_type': [u'short text']}
        request = self.factory.post(url, data)
        request.user = self.user

        response = AddQuestionToQuizView.as_view()(request, pk=self.quiz.id)
        self.assertEquals(response.status_code, 302)
        self.assertEquals(self.quiz.question_set.count(), 2)

        question = self.quiz.question_set.get(text="the text")
        self.assertEquals(question.intro_text, 'the intro')
        self.assertEquals(question.explanation, 'the explanation')
        self.assertEquals(question.question_type, 'short text')

    def test_edit_question(self):
        question = self.quiz.question_set.all()[0]
        url = "/quizblock/edit_question/%s/" % question.id

        data = {u'text': [u'alt text'],
                u'intro_text': [u'alt intro'],
                u'explanation': [u'alt explanation'],
                u'question_type': [u'short text']}
        request = self.factory.post(url, data)
        request.user = self.user

        response = EditQuestionView.as_view()(request, pk=question.id)
        self.assertEquals(response.status_code, 302)

        question = self.quiz.question_set.all()[0]
        self.assertEquals(question.intro_text, 'alt intro')
        self.assertEquals(question.explanation, 'alt explanation')
        self.assertEquals(question.question_type, 'short text')

    def test_add_answer_to_question(self):
        question = self.quiz.question_set.all()[0]
        self.assertEquals(question.answer_set.count(), 2)
        url = '/quizblock/edit_question/%s/add_answer/' % question.id

        data = {u'explanation': [u'the explanation'],
                u'value': [u'2'],
                u'label': [u'Maybe']}
        request = self.factory.post(url, data)
        request.user = self.user

        response = AddAnswerToQuestionView.as_view()(request, pk=question.id)
        self.assertEquals(response.status_code, 302)

        self.assertEquals(question.answer_set.count(), 3)
        answer = question.answer_set.get(label='Maybe')
        self.assertEquals(answer.explanation, 'the explanation')
        self.assertEquals(answer.value, '2')

    def test_add_answer_to_question_emptylabel(self):
        question = self.quiz.question_set.all()[0]
        url = '/quizblock/edit_question/%s/add_answer/' % question.id

        data = {u'explanation': [u'the explanation'],
                u'value': [u'Maybe']}
        request = self.factory.post(url, data)
        request.user = self.user

        response = AddAnswerToQuestionView.as_view()(request, pk=question.id)
        self.assertEquals(response.status_code, 302)

        self.assertEquals(question.answer_set.count(), 3)
        question.answer_set.get(label='Maybe')

    def test_delete_question(self):
        question = self.quiz.question_set.all()[0]
        url = '/quizblock/delete_question/%s/' % question.id

        request = self.factory.post(url, {})
        request.user = self.user

        response = DeleteQuestionView.as_view()(request, pk=question.id)
        self.assertEquals(response.status_code, 302)

        self.assertEquals(self.quiz.question_set.count(), 0)

    def test_reorder_answers(self):
        question = self.quiz.question_set.all()[0]

        yes = Answer.objects.get(label='Yes')
        self.assertEquals(yes.display_number(), 1)
        no = Answer.objects.get(label='No')
        self.assertEquals(no.display_number(), 2)

        args = 'answer_0=%s&answer_1=%s' % (yes.id, no.id)
        url = '/quizblock/reorder_answers/%s/?%s' % (question.id, args)
        request = self.factory.post(url, {})
        request.user = self.user

        response = ReorderAnswersView.as_view()(request, pk=question.id)
        self.assertEquals(response.status_code, 200)

        yes = Answer.objects.get(label='Yes')
        self.assertEquals(yes.display_number(), 1)
        no = Answer.objects.get(label='No')
        self.assertEquals(no.display_number(), 2)

    def test_reorder_questions(self):
        self.short_text = Question.objects.create(
            quiz=self.quiz, text='short text', question_type='short text')

        self.assertEqual(self.single_answer.display_number(), 1)
        self.assertEqual(self.short_text.display_number(), 2)

        args = 'question_0=%s&question_1=%s' % (self.short_text.id,
                                                self.single_answer.id)
        url = '/quizblock/reorder_questions/%s/?%s' % (self.quiz.id, args)
        request = self.factory.post(url)
        request.user = self.user

        response = ReorderQuestionsView.as_view()(request, pk=self.quiz.id)
        self.assertEquals(response.status_code, 200)

        self.short_text = Question.objects.get(text='short text')
        self.assertEqual(self.short_text.display_number(), 1)
        self.single_answer = Question.objects.get(text='single answer')
        self.assertEqual(self.single_answer.display_number(), 2)

    def test_delete_answer(self):
        question = self.quiz.question_set.all()[0]
        self.assertEquals(question.answer_set.count(), 2)
        answer = question.answer_set.all()[0]
        url = '/quizblock/delete_answer/%s/' % answer.id

        request = self.factory.post(url, {})
        request.user = self.user

        response = DeleteAnswerView.as_view()(request, pk=answer.id)
        self.assertEquals(response.status_code, 302)

        self.assertEquals(question.answer_set.count(), 1)

    def test_edit_answer(self):
        question = self.quiz.question_set.all()[0]
        answer = question.answer_set.all()[0]
        url = "/quizblock/edit_answer/%s/" % answer.id

        data = {u'explanation': [u'alt explanation'],
                u'value': [u'Possibly']}
        request = self.factory.post(url, data)
        request.user = self.user

        response = EditAnswerView.as_view()(request, pk=answer.id)
        self.assertEquals(response.status_code, 302)

        answer = question.answer_set.get(value='Possibly')
        self.assertEquals(answer.explanation, 'alt explanation')



from django.conf.urls import patterns
from .views import (
    EditQuizView, DeleteQuestionView, DeleteAnswerView,
    ReorderAnswersView, ReorderQuestionsView,
    AddQuestionToQuizView, EditQuestionView,
    AddAnswerToQuestionView, EditAnswerView,
)

urlpatterns = patterns(
    'quizblock.views',
    (r'^edit_quiz/(?P<pk>\d+)/$', EditQuizView.as_view(), {}, 'edit-quiz'),
    (r'^edit_quiz/(?P<pk>\d+)/add_question/$', AddQuestionToQuizView.as_view(),
     {}, 'add-question-to-quiz'),
    (r'^edit_question/(?P<pk>\d+)/$', EditQuestionView.as_view(), {},
     'edit-question'),
    (r'^edit_question/(?P<pk>\d+)/add_answer/$',
     AddAnswerToQuestionView.as_view(), {}, 'add-answer-to-question'),
    (r'^delete_question/(?P<pk>\d+)/$', DeleteQuestionView.as_view(), {},
     'delete-question'),
    (r'^reorder_answers/(?P<pk>\d+)/$', ReorderAnswersView.as_view(), {},
     'reorder-answer'),
    (r'^reorder_questions/(?P<pk>\d+)/$', ReorderQuestionsView.as_view(), {},
     'reorder-questions'),
    (r'^delete_answer/(?P<pk>\d+)/$', DeleteAnswerView.as_view(),
     {}, 'delete-answer'),
    (r'^edit_answer/(?P<pk>\d+)/$', EditAnswerView.as_view(),
     {}, 'edit-answer'),
)



from pagetree_export import register_class as register
import quizblock.models as quizblock
from pagetree_export.utils import asbool, sanitize


@register
class QuizExporter(object):
    block_class = quizblock.Quiz
    identifier = 'quiz'

    def exporter(self, block, xmlfile, zipfile):
        filename = "pageblocks/%s-description.txt" % block.pageblock().pk
        zipfile.writestr(filename, block.description.encode("utf8"))

        print >> (
            xmlfile,
            u"""<quiz rhetorical="%s" description_src="%s">""" % (
                block.rhetorical, filename))

        for question in block.question_set.all():
            print >> xmlfile, u"""<question type="%s">""" % (
                question.question_type)

            filename = "pageblocks/%s-%s-text.txt" % (block.pageblock().pk,
                                                      question.pk)
            zipfile.writestr(filename, question.text.encode("utf8"))
            print >> xmlfile, u"<text src='%s' />" % filename

            filename = ("pageblocks/%s-%s-explanation.txt" %
                        (block.pageblock().pk, question.pk))
            zipfile.writestr(filename, question.explanation.encode("utf8"))
            print >> xmlfile, u"<explanation src='%s' />" % filename

            filename = ("pageblocks/%s-%s-introtext.txt" %
                        (block.pageblock().pk, question.pk))
            zipfile.writestr(filename, question.intro_text.encode("utf8"))
            print >> xmlfile, u"<introtext src='%s' />" % filename

            for answer in question.answer_set.all():
                print >> xmlfile, \
                    u"""<answer label="%s" value="%s"
                    correct="%s" explanation="%s"/>""" % (
                    sanitize(answer.label),
                    sanitize(answer.value),
                    sanitize(answer.explanation),
                    answer.correct)

            print >> xmlfile, "</question>"
        print >> xmlfile, "</quiz>"

    def importer(self, node, zipfile):
        children = node.getchildren()
        assert len(children) == 1 and children[0].tag == "quiz"
        rhetorical = asbool(children[0].get("rhetorical"))
        path = children[0].get("description_src")
        description = zipfile.read(path)
        q = quizblock.Quiz(rhetorical=rhetorical, description=description)
        q.save()

        for child in children[0].iterchildren():
            assert child.tag == "question"
            type = child.get("type")

            (text, explanation, introtext,
             answers) = (child.getchildren()[:3] + [child.getchildren()[3:]])
            path = text.get("src")
            text = zipfile.read(path)
            path = explanation.get("src")
            explanation = zipfile.read(path)
            path = introtext.get("src")
            introtext = zipfile.read(path)
            question = quizblock.Question(
                quiz=q, text=text, question_type=type,
                explanation=explanation,
                intro_text=introtext)
            question.save()

            for answer in answers:
                label = answer.get("label")
                value = answer.get("value")
                correct = asbool(answer.get("correct"))
                explanation = answer.get("explanation")
                answer = quizblock.Answer(
                    question=question,
                    value=value,
                    label=label,
                    correct=correct,
                    explanation=explanation)
                answer.save()

        return q



from pagetree.models import Hierarchy
from pagetree.reports import PagetreeReport
from quizblock.models import Quiz, Question, Answer, Submission, \
    QuestionColumn, Response
from pagetree.tests.factories import UserFactory, PagetreeTestCase


class QuestionColumnTest(PagetreeTestCase):

    def setUp(self):
        super(QuestionColumnTest, self).setUp()

        quiz = Quiz()
        quiz.save()

        self.section = self.hierarchy_one.get_root().get_next()
        self.section.append_pageblock('Quiz', '', content_object=quiz)

        self.user = UserFactory()
        self.user2 = UserFactory()

        self.single_answer = Question.objects.create(
            quiz=quiz, text='single answer', question_type='single choice')
        self.single_answer_one = Answer.objects.create(
            question=self.single_answer, label='Yes', value='1')
        self.single_answer_two = Answer.objects.create(
            question=self.single_answer, label='No', value='0')

        self.multiple_answer = Question.objects.create(
            quiz=quiz, text='multiple answer', question_type='multiple choice')
        self.multiple_answer_one = Answer.objects.create(
            question=self.multiple_answer, label='Yes', value='1')
        self.multiple_answer_two = Answer.objects.create(
            question=self.multiple_answer, label='No', value='0')

        self.short_text = Question.objects.create(
            quiz=quiz, text='short text', question_type='short text')

        self.long_text = Question.objects.create(
            quiz=quiz, text='long text', question_type='long text')

        self.submission = Submission.objects.create(quiz=quiz, user=self.user)

        self.report = PagetreeReport()

    def test_clean_header_empty(self):
        self.assertEqual(QuestionColumn.clean_header(''), '')

    def test_clean_header(self):
        self.assertEqual(QuestionColumn.clean_header('<<<<foo>>>>'), 'foo')

    def test_single_answer(self):
        Response.objects.create(submission=self.submission,
                                question=self.single_answer,
                                value='0')

        column = QuestionColumn(self.hierarchy_one, self.single_answer)

        # identifier
        identifier = '%s_%s' % (self.hierarchy_one.id, self.single_answer.id)
        self.assertEquals(column.identifier(), identifier)

        # key row
        key_row = ['one', identifier, 'Quiz', 'single choice', 'single answer']
        self.assertEquals(column.metadata(), key_row)

        # user value
        answer = self.single_answer.answer_set.get(value='0')
        self.assertEquals(column.user_value(self.user), str(answer.id))
        self.assertEquals(column.user_value(self.user2), None)

    def test_multiple_answer(self):
        Response.objects.create(submission=self.submission,
                                question=self.multiple_answer, value='0')
        Response.objects.create(submission=self.submission,
                                question=self.multiple_answer, value='1')

        a = self.multiple_answer.answer_set.get(value='1')
        column = QuestionColumn(self.hierarchy_one, self.multiple_answer, a)

        # identifier
        identifier = '%s_%s_%s' % (
            self.hierarchy_one.id, self.multiple_answer.id, a.id)
        self.assertEquals(column.identifier(), identifier)

        # key row
        identifier = '%s_%s' % (self.hierarchy_one.id, self.multiple_answer.id)
        key_row = ['one', identifier, 'Quiz', 'multiple choice',
                   'multiple answer', a.id, a.label]
        self.assertEquals(column.metadata(), key_row)

        # user value
        self.assertEquals(column.user_value(self.user), str(a.id))
        self.assertEquals(column.user_value(self.user2), None)

    def test_short_text(self):
        Response.objects.create(submission=self.submission,
                                question=self.short_text, value='yes')

        column = QuestionColumn(self.hierarchy_one, self.short_text)

        # identifier
        identifier = '%s_%s' % (self.hierarchy_one.id, self.short_text.id)
        self.assertEquals(column.identifier(), identifier)

        # key row
        key_row = ['one', identifier, 'Quiz', 'short text', 'short text']
        self.assertEquals(column.metadata(), key_row)

        # user value
        self.assertEquals(column.user_value(self.user), 'yes')
        self.assertEquals(column.user_value(self.user2), None)

    def test_long_text(self):
        Response.objects.create(submission=self.submission,
                                question=self.long_text,
                                value='a longer response')

        column = QuestionColumn(self.hierarchy_one, self.long_text)

        # identifier
        identifier = '%s_%s' % (self.hierarchy_one.id, self.long_text.id)
        self.assertEquals(column.identifier(), identifier)

        # key row
        key_row = ['one', identifier, 'Quiz', 'long text', 'long text']
        self.assertEquals(column.metadata(), key_row)

        # user value
        self.assertEquals(column.user_value(self.user), 'a longer response')
        self.assertEquals(column.user_value(self.user2), None)

    def test_report_metadata_columns(self):
        hierarchies = Hierarchy.objects.filter(name="one")
        columns = self.report.metadata_columns(hierarchies)
        self.assertEquals(len(columns), 7)

        self.assertEquals(columns[0].question, self.single_answer)
        self.assertEquals(columns[0].answer, self.single_answer_one)

        self.assertEquals(columns[1].question, self.single_answer)
        self.assertEquals(columns[1].answer, self.single_answer_two)

        self.assertEquals(columns[2].question, self.multiple_answer)
        self.assertEquals(columns[2].answer, self.multiple_answer_one)

        self.assertEquals(columns[3].question, self.multiple_answer)
        self.assertEquals(columns[3].answer, self.multiple_answer_two)

        self.assertEquals(columns[4].question, self.short_text)

        self.assertEquals(columns[5].question, self.long_text)

    def test_report_value_columns(self):
        hierarchies = Hierarchy.objects.filter(name="one")
        columns = self.report.value_columns(hierarchies)
        self.assertEquals(len(columns), 6)

        self.assertEquals(columns[0].question, self.single_answer)
        self.assertIsNone(columns[0].answer)

        self.assertEquals(columns[1].question, self.multiple_answer)
        self.assertEquals(columns[1].answer, self.multiple_answer_one)

        self.assertEquals(columns[2].question, self.multiple_answer)
        self.assertEquals(columns[2].answer, self.multiple_answer_two)

        self.assertEquals(columns[3].question, self.short_text)

        self.assertEquals(columns[4].question, self.long_text)



from django import template
from quizblock.models import Response, Submission

register = template.Library()


class GetQuestionResponseNode(template.Node):
    def __init__(self, question, var_name):
        self.question = question
        self.var_name = var_name

    def render(self, context):
        q = context[self.question]
        u = context['request'].user
        quiz = q.quiz
        r = Submission.objects.filter(quiz=quiz, user=u).order_by("-submitted")
        if r.count() == 0:
            return ''
        submission = r[0]
        r = Response.objects.filter(question=q, submission=submission)
        if r.count() > 0:
            context[self.var_name] = r[0]
        else:
            context[self.var_name] = None
        return ''


@register.tag('getquestionresponse')
def getquestionresponse(parser, token):
    question = token.split_contents()[1:][0]
    var_name = token.split_contents()[1:][2]
    return GetQuestionResponseNode(question, var_name)


class IfAnswerInNode(template.Node):
    def __init__(self, response, answer, nodelist_true, nodelist_false=None):
        self.nodelist_true = nodelist_true
        self.nodelist_false = nodelist_false
        self.response = response
        self.answer = answer

    def render(self, context):
        r = context[self.response]
        q = r.question
        a = context[self.answer]
        u = context['request'].user
        if a.value in [resp.value for resp in q.user_responses(u)]:
            return self.nodelist_true.render(context)
        else:
            return self.nodelist_false.render(context)


@register.tag('ifanswerin')
def ifanswerin(parser, token):
    response = token.split_contents()[1:][0]
    answer = token.split_contents()[1:][1]
    nodelist_true = parser.parse(('else', 'endifanswerin'))
    token = parser.next_token()
    if token.contents == 'else':
        nodelist_false = parser.parse(('endifanswerin',))
        parser.delete_first_token()
    else:
        nodelist_false = None
    return IfAnswerInNode(response, answer, nodelist_true, nodelist_false)



# Copyright (c) 2010, Columbia Center For New Media Teaching And Learning (CCNMTL)
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of the CCNMTL nor the
#       names of its contributors may be used to endorse or promote products
#       derived from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY CCNMTL ``AS IS'' AND ANY
# EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL <copyright holder> BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

from setuptools import setup, find_packages

setup(
    name="django-quizblock",
    version="1.0.0",
    author="Anders Pearson",
    author_email="anders@columbia.edu",
    url="",
    description="Basic Quiz PageBlock",
    long_description="Basic Quiz PageBlocks (see django-pagetree and django-pageblocks)",
    install_requires = ['django-pagetree >= 0.14.5'],
    scripts = [],
    license = "BSD",
    platforms = ["any"],
    zip_safe=False,
    package_data = {'' : ['*.*']},
    packages=find_packages(),
    test_suite='nose.collector',
    include_package_data=True,

    )



from django.test import TestCase
from quizblock.models import Quiz, Question, Answer, Submission
from quizblock.models import Response
from quizblock.templatetags.getresponse import GetQuestionResponseNode
from quizblock.templatetags.getresponse import IfAnswerInNode
from django.contrib.auth.models import User


class FakeRequest(object):
    pass


class GetQuestionResponseNodeTest(TestCase):
    def test_render(self):
        q = Quiz.objects.create()
        question = Question.objects.create(
            quiz=q, text="a question", question_type='long text')
        u = User.objects.create(username="testuser")
        submission = Submission.objects.create(
            quiz=q,
            user=u)
        response = Response.objects.create(
            submission=submission,
            question=question,
            value="a long text response")
        n = GetQuestionResponseNode('question', "foo")
        r = FakeRequest()
        r.user = u
        context = dict(request=r, question=question)
        out = n.render(context)
        self.assertEqual(out, '')
        self.assertTrue('foo' in context)
        self.assertEqual(context['foo'], response)

    def test_render_no_submission(self):
        q = Quiz.objects.create()
        question = Question.objects.create(
            quiz=q, text="a question", question_type='long text')
        u = User.objects.create(username="testuser")
        n = GetQuestionResponseNode('question', "foo")
        r = FakeRequest()
        r.user = u
        context = dict(request=r, question=question)
        out = n.render(context)
        self.assertEqual(out, '')
        self.assertFalse('foo' in context)

    def test_render_submission_but_no_response(self):
        q = Quiz.objects.create()
        question = Question.objects.create(
            quiz=q, text="a question", question_type='long text')
        u = User.objects.create(username="testuser")
        Submission.objects.create(
            quiz=q,
            user=u)
        n = GetQuestionResponseNode('question', "foo")
        r = FakeRequest()
        r.user = u
        context = dict(request=r, question=question)
        out = n.render(context)
        self.assertEqual(out, '')
        self.assertTrue('foo' in context)
        self.assertEqual(context['foo'], None)


class MockNodeList(object):
    def __init__(self):
        self.rendered = False

    def render(self, c):
        self.rendered = True


class IfAnswerInTest(TestCase):
    def test_render_true(self):
        q = Quiz.objects.create()
        question = Question.objects.create(
            quiz=q, text="a question", question_type='single choice')
        answer = Answer.objects.create(question=question,
                                       value='1', label='one')
        u = User.objects.create(username="testuser")
        submission = Submission.objects.create(
            quiz=q,
            user=u)
        response = Response.objects.create(
            submission=submission,
            question=question,
            value="1")
        nl1 = MockNodeList()
        nl2 = MockNodeList()
        n = IfAnswerInNode('response', 'answer', nl1, nl2)
        r = FakeRequest()
        r.user = u
        context = dict(request=r, question=question, answer=answer,
                       response=response)
        out = n.render(context)
        self.assertEqual(out, None)
        self.assertTrue(nl1.rendered)
        self.assertFalse(nl2.rendered)

    def test_render_false(self):
        q = Quiz.objects.create()
        question = Question.objects.create(
            quiz=q, text="a question", question_type='single choice')
        Answer.objects.create(
            question=question,
            value='1', label='one')
        answer2 = Answer.objects.create(
            question=question,
            value='2', label='two')
        u = User.objects.create(username="testuser")
        submission = Submission.objects.create(
            quiz=q,
            user=u)
        response = Response.objects.create(
            submission=submission,
            question=question,
            value="1")
        nl1 = MockNodeList()
        nl2 = MockNodeList()
        n = IfAnswerInNode('response', 'answer', nl1, nl2)
        r = FakeRequest()
        r.user = u
        context = dict(request=r, question=question, answer=answer2,
                       response=response)
        out = n.render(context)
        self.assertEqual(out, None)
        self.assertFalse(nl1.rendered)
        self.assertTrue(nl2.rendered)



from models import Quiz, Question, Answer
from django.http import HttpResponseRedirect, HttpResponse
from django.shortcuts import render, get_object_or_404
from django.core.urlresolvers import reverse
from django.views.generic.base import View
from django.views.generic.detail import DetailView
from django.views.generic.edit import DeleteView


class EditQuizView(DetailView):
    model = Quiz


class DeleteQuestionView(DeleteView):
    model = Question

    def get_success_url(self):
        quiz = self.object.quiz
        return reverse("edit-quiz", args=[quiz.id])


class DeleteAnswerView(DeleteView):
    model = Answer

    def get_success_url(self):
        question = self.object.question
        return reverse("edit-question", args=[question.id])


class ReorderItemsView(View):
    def post(self, request, pk):
        parent = get_object_or_404(self.parent_model, pk=pk)
        keys = [k
                for k in request.GET.keys()
                if k.startswith(self.prefix)]
        keys.sort(key=lambda x: int(x.split("_")[1]))
        items = [int(request.GET[k])
                 for k in keys if k.startswith(self.prefix)]
        self.update_order(parent, items)
        return HttpResponse("ok")


class ReorderAnswersView(ReorderItemsView):
    parent_model = Question
    prefix = "answer_"

    def update_order(self, parent, items):
        parent.update_answers_order(items)


class ReorderQuestionsView(ReorderItemsView):
    parent_model = Quiz
    prefix = "question_"

    def update_order(self, parent, items):
        parent.update_questions_order(items)


class AddQuestionToQuizView(View):
    def post(self, request, pk):
        quiz = get_object_or_404(Quiz, pk=pk)
        form = quiz.add_question_form(request.POST)
        if form.is_valid():
            question = form.save(commit=False)
            question.quiz = quiz
            question.save()
        return HttpResponseRedirect(reverse("edit-quiz", args=[quiz.id]))


class EditQuestionView(View):
    template_name = "quizblock/edit_question.html"

    def get(self, request, pk):
        question = get_object_or_404(Question, pk=pk)
        return render(
            request,
            self.template_name,
            dict(question=question, answer_form=question.add_answer_form()))

    def post(self, request, pk):
        question = get_object_or_404(Question, pk=pk)
        form = question.edit_form(request.POST)
        question = form.save(commit=False)
        question.save()
        return HttpResponseRedirect(reverse("edit-question",
                                            args=[question.id]))


class AddAnswerToQuestionView(View):
    template_name = 'quizblock/edit_question.html'

    def get(self, request, pk):
        question = get_object_or_404(Question, pk=pk)
        form = question.add_answer_form()
        return render(
            request,
            self.template_name,
            dict(question=question, answer_form=form))

    def post(self, request, pk):
        question = get_object_or_404(Question, pk=pk)
        form = question.add_answer_form(request.POST)
        if form.is_valid():
            answer = form.save(commit=False)
            answer.question = question
            if answer.label == '':
                answer.label = answer.value
            answer.save()
            return HttpResponseRedirect(reverse("edit-question",
                                                args=[question.id]))
        return render(
            request,
            self.template_name,
            dict(question=question, answer_form=form))


class EditAnswerView(View):
    template_name = 'quizblock/edit_answer.html'

    def get(self, request, pk):
        answer = get_object_or_404(Answer, pk=pk)
        form = answer.edit_form()
        return render(
            request,
            self.template_name,
            dict(answer_form=form, answer=answer))

    def post(self, request, pk):
        answer = get_object_or_404(Answer, pk=pk)
        form = answer.edit_form(request.POST)
        if form.is_valid():
            answer = form.save(commit=False)
            answer.save()
            return HttpResponseRedirect(reverse("edit-answer",
                                                args=[answer.id]))
        return render(
            request,
            self.template_name,
            dict(answer_form=form, answer=answer))
